import dart
from fwk.coloring import ccolors

def initBL(Re, Minf, CFL0, nSections, verbose=1):
    if Re<=0.:
        print(ccolors.ANSI_RED, "dart::vUtils Error : Negative Reynolds number.", Re, ccolors.ANSI_RESET)
        raise RuntimeError("Invalid parameter")
    if Minf<0.:
        print(ccolors.ANSI_RED, "dart::vUtils Error : Negative Mach number.", Minf, ccolors.ANSI_RESET)
        raise RuntimeError("Invalid parameter")
    elif Minf>=1.:
        print(ccolors.ANSI_YELLOW, "dart::vUtils Warning : (Super)sonic freestream Mach number.", Minf, ccolors.ANSI_RESET)
    if nSections < 0:
        print(ccolors.ANSI_RED, "dart::vUtils Fatal error : Negative number of sections.", nSections, ccolors.ANSI_RESET)
        raise RuntimeError("Invalid parameter")
    if not(0<=verbose<=3):
        print(ccolors.ANSI_RED, "dart::vUtils Fatal error : verbose not in valid range.", verbose, ccolors.ANSI_RESET)
        raise RuntimeError("Invalid parameter")
    
    solver = dart.ViscSolver(Re, Minf, CFL0, nSections, verbose)
    return solver


def GetSolution(iSec, vSolver):
  if iSec<0:
    raise RuntimeError("dart::viscU Invalid section number", iSec)
  
  brutSln = vSolver.GetSolution(iSec)

  sln = {'theta'    : brutSln[0],
         'H'        : brutSln[1],
         'N'        : brutSln[2],
         'ue'       : brutSln[3],
         'Ct'       : brutSln[4],
         'deltaStar': brutSln[5],
         'Ret'      : brutSln[6],
         'Cd'       : brutSln[7],
         'Cf'       : brutSln[8],
         'Hstar'    : brutSln[9],
         'Hstar2'   : brutSln[10],
         'Hk'       : brutSln[11],
         'Cteq'     : brutSln[12],
         'Us'       : brutSln[13],
         'delta'    : brutSln[14],
         'x/c'      : brutSln[15],
         'xtrT'     : vSolver.Getxtr(iSec, 0),
         'xtrB'     : vSolver.Getxtr(iSec, 1),
         'Cdt'      : vSolver.Cdt,
         'Cdf'      : vSolver.Cdf,
         'Cdp'      : vSolver.Cdp}
  return sln

def WriteFile(wData, Re, toW=['deltaStar', 'H', 'Cf', 'Ct', 'delta']):
  """Write the results in airfoil_viscous.dat
  """
  
  # Write
  print('Writing file: airfoil_viscous.dat...', end = '')
  f = open('airfoil_viscous.dat', 'w+')

  f.write('$Aerodynamic coefficients\n')
  f.write('             Re              Cd             Cdp             Cdf         xtr_top         xtr_bot\n')
  f.write('{0:15.6f} {1:15.6f} {2:15.6f} {3:15.6f} {4:15.6f} {5:15.6f}\n'.format(Re, wData['Cdt'], wData['Cdp'],
                                                                                  wData['Cdf'], wData['xtrT'],
                                                                                  wData['xtrB']))
  f.write('$Boundary layer variables\n')
  f.write('            x/c')

  for s in toW:
    f.write(' {0:>15s}'.format(s))
  f.write('\n')

  for i in range(len(wData['x/c'])):
    f.write('{0:15.6f}'.format(wData['x/c'][i]))
    for s in toW:
      f.write(' {0:15.6f}'.format(wData[s][i]))
    f.write('\n')

  f.close()
  print('done.')


def PlotVars(wData, var='H'):
  from matplotlib import pyplot as plt

  nUpper = len(wData['x/c'])
  for i in range(len(wData['x/c'])):
    if wData['x/c'][i] == 1.0:
      nUpper = i+1
      break

  # [0   1          2      3  4  5   6   7      8       9  10  11  12     13     14  15    16]  
  # [xx, x, deltaStar, theta, H, N, Ue, Hk, Hstar, Hstar2, Cf, Cd, Ct, delta, VtExt, Me, Rhoe]
  
  plt.plot(wData['x/c'][:nUpper], wData[var][:nUpper], color='#0072BD', linewidth=3)
  plt.plot(wData['x/c'][nUpper:], wData[var][nUpper:], color='#D95319', linewidth=3)
  plt.axvline(x=wData['xtrTop'], linestyle="--", color='#0072BD')
  plt.axvline(x=wData['xtrBot'], linestyle="--", color='#D95319')
  plt.xlabel('$x/c$')
  plt.ylabel(var)
  plt.xlim([0,2])
  plt.legend(['Upper side', 'Lower side'], frameon=False)
  plt.show()

def PlotPolar(coeffTable):
  from matplotlib import pyplot as plt

  plt.plot(coeffTable[0,:], coeffTable[1,:], 'x-')
  plt.xlabel('$AoA$ (deg)')
  plt.ylabel('$c_l$ (-)')
  plt.show()
  plt.plot(coeffTable[1,:], coeffTable[2,:], 'x-')
  plt.xlabel('$c_l$ (-)')
  plt.ylabel('$c_d$ (-)')
  plt.show()